#ifndef GRID_HPP
#define GRID_HPP

#include "Common.hpp"

template<typename T>
class Grid
{
public:
    Grid() : m_data(), m_width(0), m_height(0) {}

    Grid(size_t width, size_t height) : m_data(), m_width(0), m_height(0)
    {
        resize(width, height);
    }

    Grid(Grid&& other) :
        m_data(std::move(other.m_data)),
        m_width(other.m_width),
        m_height(other.m_height)
    {
        other.m_width = 0;
        other.m_height = 0;
    }

    ~Grid() {}

    Grid& operator=(Grid&& other)
    {
        if (this != &other)
        {
            m_data = std::move(other.m_data);
            m_width = other.m_width;
            m_height = other.m_height;
            other.m_width = 0;
            other.m_height = 0;
        }
        return *this;
    }

    size_t getWidth() const { return m_width; }
    size_t getHeight() const { return m_height; }

    void resize(size_t width, size_t height)
    {
        m_data.reset(width != 0 && height != 0 ?
                     new T[width * height]() :
                     nullptr);
        m_width = width;
        m_height = height;
    }

    const T& operator()(size_t x, size_t y) const { return m_data[y * m_width + x]; }
    T& operator()(size_t x, size_t y) { return m_data[y * m_width + x]; }

    void swap(Grid& other)
    {
        using std::swap;
        swap(m_data, other.m_data);
        swap(m_width, other.m_width);
        swap(m_height, other.m_height);
    }

private:
    std::unique_ptr<T[]> m_data;
    size_t m_width;
    size_t m_height;

private:
    Grid(const Grid& other) = delete;
    Grid& operator=(const Grid& other) = delete;
};

template<class T>
void swap(Grid<T>& x, Grid<T>& y)
{
    x.swap(y);
}

#endif
