#include "Moves.hpp"
#include "Game.hpp"
#include "Transform.hpp"
#include "Move.hpp"
#include <queue>
#include <algorithm>

using std::vector;
using std::set;
using std::map;
using std::queue;

MoveFinder::MoveFinder()
{
}

void MoveFinder::findMoves(const Game& game, vector<Move>* pMoves)
{
    const Command commands[] = {
        Command::SW,
        Command::SE,
        Command::W,
        Command::E,
        Command::CW,
        Command::CCW
    };

    m_moveIndex.clear();
    m_unitIndex.clear();
    m_moves.clear();
    m_units.clear();

    queue<UnitRec> unitQueue;
    {
        UnitRec rec;
        rec.m_unit = game.m_curUnit;
        rec.m_prevUnit = (size_t)-1;
        rec.m_command = Command::E; // unused
        unitQueue.push(std::move(rec));
    }

    while (!unitQueue.empty())
    {
        UnitRec curRec(std::move(unitQueue.front()));
        unitQueue.pop();
        if (m_unitIndex.count(curRec.m_unit))
        {
            continue;
        }
        size_t curIndex = m_units.size();
        m_units.push_back(curRec);
        m_unitIndex[curRec.m_unit] = curIndex;
        for (uint32_t iCommand = 0; iCommand < 6; iCommand++)
        {
            Command command = commands[iCommand];
            Unit transformedUnit(curRec.m_unit);
            transformUnit(command, &transformedUnit);
            if (game.isValid(transformedUnit))
            {
                UnitRec rec;
                rec.m_unit = std::move(transformedUnit);
                rec.m_prevUnit = curIndex;
                rec.m_command = command;
                unitQueue.push(std::move(rec));
            }
            else
            {
                UnitRec rec;
                rec.m_unit = curRec.m_unit;
                rec.m_unit.m_pivot = Cell();
                rec.m_prevUnit = curIndex;
                rec.m_command = command;
                if (!m_moveIndex.count(rec.m_unit))
                {
                    size_t iMoveIndex = m_moves.size();
                    m_moves.push_back(rec);
                    m_moveIndex[std::move(rec.m_unit)] = iMoveIndex;
                }
            }
        }
    }

    pMoves->resize(m_moves.size());
    for (size_t i = 0; i < m_moves.size(); i++)
    {
        (*pMoves)[i].m_members = m_moves[i].m_unit.m_members;
    }
}

void MoveFinder::getCommands(const Move& move, vector<Command>* pCommands) const
{
    vector<Command>& commands = *pCommands;
    commands.clear();
    Unit unit;
    unit.m_members = move.m_members;
    auto it = m_moveIndex.find(unit);
    if (it == m_moveIndex.end())
    {
        fprintf(stderr, "MoveFinder::getCommands: Move not found\n");
        exit(1);
    }

    const UnitRec* pRec = &m_moves[it->second];
    while (pRec->m_prevUnit != (size_t)-1)
    {
        commands.push_back(pRec->m_command);
        pRec = &m_units[pRec->m_prevUnit];
    }

    std::reverse(commands.begin(), commands.end());
}
