#ifndef TRANSFORM_HPP
#define TRANSFORM_HPP

#include "Common.hpp"
#include "Command.hpp"
#include "Cell.hpp"
#include "Unit.hpp"

inline Cell transformCell(Command command, Cell pivot, Cell cell)
{
    switch (command)
    {
    case Command::E:
        return Cell(cell.m_x + 1, cell.m_y);
    case Command::W:
        return Cell(cell.m_x - 1, cell.m_y);
    case Command::SE:
        return Cell(cell.m_x + (int32_t)((uint32_t)cell.m_y & 1), cell.m_y + 1);
    case Command::SW:
        return Cell(cell.m_x - 1 + (int32_t)((uint32_t)cell.m_y & 1), cell.m_y + 1);
    case Command::CW:
    {
        int32_t xPivot = pivot.m_x - (pivot.m_y >> 1);
        int32_t yPivot = pivot.m_y;
        int32_t x = cell.m_x - (cell.m_y >> 1) - xPivot;
        int32_t y = cell.m_y - yPivot;
        //int32_t a = x;
        int32_t b = -x - y;
        int32_t c = y;
        int32_t aNew = -c;
        //int32_t bNew = -a;
        int32_t cNew = -b;
        int32_t xNew = aNew;
        int32_t yNew = cNew;
        return Cell(xNew + xPivot + ((yNew + yPivot) >> 1), yNew + yPivot);
    }
    case Command::CCW:
    {
        int32_t xPivot = pivot.m_x - (pivot.m_y >> 1);
        int32_t yPivot = pivot.m_y;
        int32_t x = cell.m_x - (cell.m_y >> 1) - xPivot;
        int32_t y = cell.m_y - yPivot;
        int32_t a = x;
        int32_t b = -x - y;
        //int32_t c = y;
        int32_t aNew = -b;
        //int32_t bNew = -c;
        int32_t cNew = -a;
        int32_t xNew = aNew;
        int32_t yNew = cNew;
        return Cell(xNew + xPivot + ((yNew + yPivot) >> 1), yNew + yPivot);
    }
    default:
        return cell;
    }
}

inline void transformUnit(Command command, Unit* pUnit)
{
    Unit& unit = *pUnit;
    for (auto& member : unit.m_members)
    {
        member = transformCell(command, unit.m_pivot, member);
    }
    unit.m_pivot = transformCell(command, unit.m_pivot, unit.m_pivot);
    std::sort(unit.m_members.begin(), unit.m_members.end());
}

#endif
