#ifndef UTILS_HPP
#define UTILS_HPP

#include "Common.hpp"

class Cleanup
{
public:
    explicit Cleanup(std::function<void()> func = nullptr) :
        m_func(std::move(func))
    {
    }

    Cleanup(Cleanup&& other) :
        m_func(std::move(other.m_func))
    {
        other.m_func = nullptr;
    }

    ~Cleanup()
    {
        if (m_func)
        {
            m_func();
        }
    }

    Cleanup& operator=(Cleanup&& other)
    {
        if (this != &other)
        {
            m_func = std::move(other.m_func);
            other.m_func = nullptr;
        }
        return *this;
    }

    void reset(std::function<void()> func = nullptr)
    {
        m_func = std::move(func);
    }

    void cleanup()
    {
        if (m_func)
        {
            m_func();
        }
        m_func = nullptr;
    }

private:
    std::function<void()> m_func;

private:
    Cleanup(const Cleanup& other) = delete;
    Cleanup& operator=(const Cleanup& other) = delete;
};

template<typename A, typename B>
auto mod(A a, B b) -> decltype(a % b)
{
    auto r = a % b;
    if (r < 0)
    {
        r += b;
    }
    return r;
}

void sleepS(uint32_t timeS);
void sleepMS(uint32_t timeMS);

uint64_t getTicksS();
uint64_t getTicksMS();

std::string strprintf(const char* format, ...);
std::string vstrprintf(const char* format, va_list ap);

int strcmpNoCase(const char* s1, const char* s2);

#endif
