#include "Common.hpp"
#include "Problem.hpp"
#include "Solution.hpp"
#include "Parser.hpp"
#include "FileUtils.hpp"
#include "CalcScore.hpp"

using std::string;
using std::vector;

void usage()
{
    printf("Usage:\n");
    printf("  score <solutions.json> <phrases.txt> <problem.json>...\n");
}

int main(int argc, char* argv[])
{
    bool gotSolutionsFile = false;
    string argSolutionsFile;
    bool gotPhraseFile = false;
    string argPhraseFile;
    vector<string> argProblemFiles;

    int iArg = 1;
    while (iArg < argc)
    {
        string strArg = argv[iArg];
        iArg++;

        if (!gotSolutionsFile)
        {
            gotSolutionsFile = true;
            argSolutionsFile = strArg;
            continue;
        }

        if (!gotPhraseFile)
        {
            gotPhraseFile = true;
            argPhraseFile = strArg;
            continue;
        }

        argProblemFiles.push_back(strArg);
    }

    if (!gotSolutionsFile ||
        !gotPhraseFile ||
        argProblemFiles.empty())
    {
        usage();
        return 1;
    }

    string msg;

    vector<Problem> problems;
    for (auto& strProblemFile : argProblemFiles)
    {
        problems.emplace_back();
        auto& problem = problems.back();

        if (!loadProblem(strProblemFile, &problem, &msg))
        {
            printf("%s\n", msg.c_str());
            return 1;
        }
    }

    vector<Solution> solutions;
    if (!loadSolutions(argSolutionsFile, &solutions, &msg))
    {
        printf("%s\n", msg.c_str());
        return 1;
    }

    vector<string> phrases;
    {
        FILE* f = fopen(argPhraseFile.c_str(), "rb");
        if (!f)
        {
            printf("Error opening file %s\n", argPhraseFile.c_str());
            return 1;
        }

        vector<string> lines;
        readLines(f, &phrases);
        fclose(f);
    }

    for (size_t iProblem = 0; iProblem < problems.size(); iProblem++)
    {
        auto& problem = problems[iProblem];

        vector<uint64_t> scores;

        for (auto seed : problem.m_sourceSeeds)
        {
            size_t iSolution = 0;
            for (iSolution = 0; iSolution < solutions.size(); iSolution++)
            {
                if (solutions[iSolution].m_problemId == problem.m_id &&
                    solutions[iSolution].m_seed == seed)
                {
                    break;
                }
            }
            if (iSolution == solutions.size())
            {
                printf("Solution not found\n");
                return 1;
            }
            const Solution& solution = solutions[iSolution];

            uint64_t score = 0;
            calcScore(problem, phrases, seed, solution, &score);
            scores.push_back(score);
        }

        uint64_t totalScore = 0;
        for (auto score : scores)
        {
            totalScore += score;
        }
        uint64_t averageScore = totalScore / scores.size();
        printf("%s:", argProblemFiles[iProblem].c_str());
        printf("   %" PRIu64 "  ", averageScore);
        for (auto score : scores)
        {
            printf(" %" PRIu64 "", score);
        }
        printf("\n");
    }

    return 0;
}
