#include "Common.hpp"
#include "Source.hpp"
#include "Problem.hpp"
#include "Parser.hpp"
#include "Utils.hpp"
#include "Ansi.hpp"
#include "Game.hpp"
#include "Moves.hpp"
#include "Move.hpp"

using std::string;
using std::vector;

int main(int argc, char* argv[])
{
    Ansi ansi;

#if 0
    uint32_t seed = 17;
    Source source(seed);
    printf("%" PRIu32 ":", seed);
    for (uint32_t i = 0; i < 10; i++)
    {
        printf(" %" PRIu32 "", source.next());
    }
    printf("\n");
#endif

    for (uint32_t iProblem = 0; iProblem < 24; iProblem++)
    {
        string strProblemFile = strprintf("../problems/problem_%" PRIu32 ".json", iProblem);
        printf("\n");
        printf("%s:\n", strProblemFile.c_str());

        string msg;
        Problem problem;
        if (!loadProblem(strProblemFile, &problem, &msg))
        {
            printf("%s\n", msg.c_str());
            return 1;
        }

#if 1
        printf("id: %" PRId64 "\n", problem.m_id);
        printf("units: |%" PRIuZ "|\n", problem.m_units.size());
        for (auto& unit : problem.m_units)
        {
            printf("  (%" PRId32 ", %" PRId32 "): ",
                   unit.m_pivot.m_x, unit.m_pivot.m_y);
            for (auto cell : unit.m_members)
            {
                printf("(%" PRId32 ", %" PRId32 ") ", cell.m_x, cell.m_y);
            }
            printf("\n");
        }
        printf("width: %" PRIu32 "\n", problem.m_width);
        printf("height: %" PRIu32 "\n", problem.m_height);
        printf("filled: |%" PRIuZ "|\n", problem.m_filled.size());
        printf("  ");
        for (auto cell : problem.m_filled)
        {
            printf("(%" PRId32 ", %" PRId32 ") ", cell.m_x, cell.m_y);
        }
        printf("\n");
        printf("sourceLength: %" PRIu64 "\n", problem.m_sourceLength);
        printf("sourceSeeds: |%" PRIuZ "|\n", problem.m_sourceSeeds.size());
        printf("  ");
        for (auto seed : problem.m_sourceSeeds)
        {
            printf("%" PRIu32 " ", seed);
        }
        printf("\n");
#endif

#if 1
        Game game;
        game.init(problem, vector<string>(), problem.m_sourceSeeds[0]);
        printGame(ansi, game);
#endif

#if 0
        vector<Move> moves;
        MoveFinder moveFinder;
        moveFinder.findMoves(game, &moves);
        printf("Moves:\n");
        for (auto& move : moves)
        {
            printf("  ");
            for (auto cell : move.m_members)
            {
                printf("(%" PRId32 ", %" PRId32 ") ", cell.m_x, cell.m_y);
            }
            printf("\n");
        }
#endif
    }

    return 0;
}
