#include "Common.hpp"
#include "Source.hpp"
#include "Problem.hpp"
#include "Solution.hpp"
#include "Parser.hpp"
#include "Utils.hpp"
#include "Ansi.hpp"
#include "Game.hpp"
#include "Input.hpp"
#include "FileUtils.hpp"

using std::string;
using std::vector;

void usage()
{
    printf("Usage:\n");
    printf("  vis <problem.json> <solutions.json> [<phrases.txt>] [<seed>]\n");
    printf("Keys:\n");
    printf("  Space:      Perform one command\n");
    printf("  Q or Esc:   Quit\n");
}

int main(int argc, char* argv[])
{
    if (argc != 3 && argc != 4)
    {
        usage();
        return 1;
    }

    string strProblemFile(argv[1]);
    string strSolutionsFile(argv[2]);
    bool gotPhraseFile = (argc == 4);
    string strPhraseFile;
    if (gotPhraseFile)
    {
        strPhraseFile = argv[3];
    }
    bool gotSeed = (argc == 5);
    uint32_t argSeed = 0;
    if (gotSeed)
    {
        argSeed = atol(argv[4]);
    }

    string msg;

    Problem problem;
    if (!loadProblem(strProblemFile, &problem, &msg))
    {
        printf("%s\n", msg.c_str());
        return 1;
    }

    vector<Solution> solutions;
    if (!loadSolutions(strSolutionsFile, &solutions, &msg))
    {
        printf("%s\n", msg.c_str());
        return 1;
    }

    vector<string> phrases;
    if (gotPhraseFile)
    {
        FILE* f = fopen(strPhraseFile.c_str(), "rb");
        if (!f)
        {
            printf("Error opening file %s\n", strPhraseFile.c_str());
            return 1;
        }

        readLines(f, &phrases);
        fclose(f);
    }

    uint32_t seed = gotSeed ? argSeed : problem.m_sourceSeeds[0];
    size_t iSolution = 0;
    for (iSolution = 0; iSolution < solutions.size(); iSolution++)
    {
        if (solutions[iSolution].m_problemId == problem.m_id &&
            solutions[iSolution].m_seed == seed)
        {
            break;
        }
    }
    if (iSolution == solutions.size())
    {
        printf("Solution not found\n");
        return 1;
    }
    const Solution& solution = solutions[iSolution];

    vector<Command> commands;
    for (size_t i = 0; i < solution.m_chars.size(); i++)
    {
        Command command = Command::E;
        bool bIllegal = false;
        if (!charToCommand(solution.m_chars[i], &command, &bIllegal))
        {
            if (bIllegal)
            {
                printf("Illegal character '%c'\n", solution.m_chars[i]);
                return 1;
            }
            else
            {
                continue;
            }
        }
        commands.push_back(command);
    }

    Ansi ansi;
    Input input;

    Game game;
    game.init(problem, phrases, seed);
    ansi.clear();
    printGame(ansi, game);

    size_t iCommand = 0;
    while (!game.m_done && iCommand < commands.size())
    {
        Key key = Key::Invalid;
        if (!input.getKey(true, &key))
        {
            break;
        }

        bool bQuit = false;
        switch (key)
        {
        case Key::Space: break;
        case Key::Q:
        case Key::Esc:
            bQuit = true; break;
        default: continue;
        }

        if (bQuit)
        {
            break;
        }

        game.step(commands[iCommand]);
        iCommand++;

        ansi.clear();
        printGame(ansi, game);
    }

    if (game.m_done && iCommand < commands.size())
    {
        printf("Extra commands at end of solution!\n");
    }
    else if (!game.m_done && iCommand == commands.size())
    {
        printf("Commands end before game is over\n");
    }

    return 0;
}
